<?php

namespace Drupal\ip_info\Controller;

use Drupal\ban\BanIpManagerInterface;
use Drupal\dblog\Controller\DbLogController;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Database\Query\TableSortExtender;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Unicode;

/**
 * Controller for IP address.
 */
class IpController extends DbLogController {

  /**
   * @var \Drupal\ban\BanIpManagerInterface
   */
  protected $ipManager;

  /**
   * Constructs a new IpController object.
   *
   * @param \Drupal\ban\BanIpManagerInterface $ip_manager
   *   The ban IP manager.
   */
  public function __construct(Connection $database, ModuleHandlerInterface $module_handler, DateFormatterInterface $date_formatter, FormBuilderInterface $form_builder, BanIpManagerInterface $ip_manager) {
    parent::__construct($database, $module_handler, $date_formatter, $form_builder);
    $this->ipManager = $ip_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function overview($ip_address) {

    // Test, if the IP address is banned.
    if ($this->ipManager->isBanned($ip_address)) {
      $build['dblog_filter_form']['#prefix'] = '<p>' . $this->t('The IP address is banned.') . ' ';
      // Find the ID of the banned IP.
      $bans = $this->ipManager->findAll()->fetchAll();
      foreach ($bans as $ban) {
        if ($ban->ip == $ip_address) {
          $ban_id = $ban->iid;
          break;
        }
      }
      // Add unban link.
      if (isset($ban_id)) {
        $build['dblog_filter_form']['#prefix'] .= Link::fromTextAndUrl($this->t('Unban this IP address'), Url::fromRoute('ban.delete', ['ban_id' => $ban_id]))->toString() . '.</p>';
      } else {
        $build['dblog_filter_form']['#prefix'] .= '</p>';
      }
    } else {
      $build['dblog_filter_form']['#prefix'] = '<p>' . $this->t('The IP address is not banned.') . ' ';
      // Add ban link.
      $build['dblog_filter_form']['#prefix'] .= Link::fromTextAndUrl($this->t('Ban this IP address'), Url::fromRoute('ban.admin_page', ['default_ip' => $ip_address]))->toString() . '.</p>';}

    // DNS Lookup.
    $lookup_services = [
      'ipinfo.io' => 'https://ipinfo.io/' . $ip_address,
      'iplocation.net' => 'https://www.iplocation.net/ip-lookup/' . $ip_address,
      'ip2location.com' => 'https://www.ip2location.com/demo/' . $ip_address,
      'keycdn.com' => 'https://tools.keycdn.com/geo?host=' . $ip_address,
    ];
    $lookup_services_links = '';
    foreach ($lookup_services as $service => $url) {
      $lookup_services_links .= '<a href="' . $url . '">' . $service . '</a>, ';
    }
    // Remove the last comma.
    $lookup_services_links = rtrim($lookup_services_links, ', ');
    $build['dblog_filter_form']['#prefix'] .= '<p>' . $this->t('Lookup the IP: ') . $lookup_services_links . '</p>';

    $rows = [];

    $classes = static::getLogLevelClassMap();

    $this->moduleHandler()->loadInclude('dblog', 'admin.inc');

    // Subtitle.
    $build['dblog_filter_form']['#prefix'] .= '<p>' . $this->t('Log messages:') . '</p>';

    $header = [
      // Icon column.
      '',
      [
        'data' => $this->t('Type'),
        'field' => 'w.type',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Date'),
        'field' => 'w.wid',
        'sort' => 'desc',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      $this->t('Message'),
      [
        'data' => $this->t('User'),
        'field' => 'ufd.name',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
    ];

    $query = $this->database->select('watchdog', 'w')
      ->extend(PagerSelectExtender::class)
      ->extend(TableSortExtender::class);
    $query->fields('w', [
      'wid',
      'uid',
      'severity',
      'type',
      'hostname',
      'timestamp',
      'message',
      'variables',
      'link',
    ]);
    $query->leftJoin('users_field_data', 'ufd', '[w].[uid] = [ufd].[uid]');

    $result = $query
      ->limit(50)
      ->condition('hostname', $ip_address)
      ->orderByHeader($header)
      ->execute();

    foreach ($result as $dblog) {
      $message = $this->formatMessage($dblog);
      if ($message && isset($dblog->wid)) {
        $title = Unicode::truncate(Html::decodeEntities(strip_tags($message)), 256, TRUE, TRUE);
        $log_text = Unicode::truncate($title, 56, TRUE, TRUE);
        // The link generator will escape any unsafe HTML entities in the final
        // text.
        $message = Link::fromTextAndUrl($log_text, new Url('dblog.event', ['event_id' => $dblog->wid], [
          'attributes' => [
            // Provide a title for the link for useful hover hints. The
            // Attribute object will escape any unsafe HTML entities in the
            // final text.
            'title' => $title,
          ],
        ]))->toString();
      }
      $username = [
        '#theme' => 'username',
        '#account' => $this->userStorage->load($dblog->uid),
      ];
      $rows[] = [
        'data' => [
          // Cells.
          ['class' => ['icon']],
          $this->t($dblog->type),
          $this->dateFormatter->format($dblog->timestamp, 'short'),
          $message,
          ['data' => $username],
          ['data' => ['#markup' => $dblog->link]],
        ],
        // Attributes for table row.
        'class' => [Html::getClass('dblog-' . $dblog->type), $classes[$dblog->severity]],
      ];
    }

    $build['dblog_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['id' => 'admin-dblog', 'class' => ['admin-dblog']],
      '#empty' => $this->t('No log messages available.'),
      '#attached' => [
        'library' => ['dblog/drupal.dblog'],
      ],
    ];
    $build['dblog_pager'] = ['#type' => 'pager'];

    return $build;

  }

  /**
   * {@inheritdoc}
   */
  public function eventDetails($event_id) {
    $build = parent::eventDetails($event_id);
    $ip_address = $build["dblog_table"]["#rows"][7][1];
    $build["dblog_table"]["#rows"][7][1] = Link::fromTextAndUrl($ip_address, Url::fromRoute('ip_info.ip', ['ip_address' => $ip_address]));
    return $build;
  }

  /**
   * Generates the page title.
   *
   * @param string $ip_address
   *   The IP address from the path.
   *
   * @return string
   *   The page title.
   */
  public function title($ip_address) {
    return 'IP Address Info for: ' . $ip_address;
  }

}
